$if not setglobal debug $offlisting offinclude

*----------------------------------------------------------------------------- 
* GAMS Source Code (resides in the ldir directory): Generic Plotting Routine
*----------------------------------------------------------------------------- 

*----------------------------------------------------------------------------- 
*       Declarations:
*----------------------------------------------------------------------------- 

$if not declared gp__1          alias(gp__1,gp__2,*);
$if not declared gp_row         set gp_row(*), gp_col(*);
$if not declared gp_r           parameter gp_r(*);
$if not declared gp_rr          alias (gp_rr,gp_row);
$if not declared gp_inp         file gp_inp /matplot.m/; 
$if not declared gp_data        file gp_data /gnuplot.dat/; 
$if not declared gp_na          scalar gp_na /na/; 
$if not declared gp_count       scalar gp_count, gp_counter;

*       If no arguments, this is solely for intialization:

$if a%1 == a $exit

$if declared %1                 $goto declared

$error PLOT ERROR: Identfier %1 is not declared.
$exit

$label declared
$if defined %1     $goto defined
$error PLOT ERROR: identfier %1 is not defined.
$exit

$label defined

$if not setglobal domain $setglobal domain 
$if not setglobal series $setglobal series
$if not "%2"=="" $setglobal domain %2

$if not setglobal gp_key     $set gp_key 0
$if not setglobal gp_xscale  $set gp_xscale 1
$if not setglobal gp_yscale  $set gp_yscale %gp_xscale%

$onuni

$if dimension 1 %1 $goto start1

$if not "%3"=="" $setglobal series %3

$if dimension 2 %1 $goto getsets
$if not %series%=="" $goto getsets
$error PLOT error: series must be specified for multidimensional plots
$exit

*       Extract row and column labels:

$label start1
gp_row(gp__1)$%1(gp__1) = yes;

$label getsets

$if not dimension 1 %1 $goto arraysets

$if "%domain%"=="" gp_row(gp__1)$abs(%1(gp__1)) = yes;
$if not "%domain%"=="" gp_row(%domain%) = yes;
$goto checkx

$label arraysets
$if not "%domain%"=="" $goto getseries

$if %series%=="" gp_row(gp__1)$sum(gp__2, abs(%1(gp__1,gp__2))) = yes;
$if not %series%=="" gp_row(gp__1)$sum(%series%, abs(%1(gp__1,%series%))) = yes;

$label getseries
$if "%series%"=="" gp_col(gp__2)$sum(gp__1, abs(%1(gp__1,gp__2))) = yes;
$if not "%domain%"=="" gp_row(%domain%) = yes;

$if not dimension 2 %1 $goto checkx
$if not "%series%"=="" gp_col(%series%) = yes;

*       See if xvalue  has been specified:

$label checkx
$if setglobal xvalue $goto xvalue

*       1. No X parameter array has been specified assign unit values:

gp_count = 0;
loop(gp_row, 
  gp_count = gp_count + 1;
  gp_r(gp_row) = gp_count; 
);
$goto write

$label xvalue

*       If the plotted item is single dimensional, skip
*       we must assume that xvalue is a parameter:

$if partype %xvalue%   $goto xparam

$if dimension 2 %1 $goto xcolumn

$error  PLOT ERROR: Cannot find parameter %xvalue% (xvalue).
$exit

*       2. X parameter values are in the data array:

$label xcolumn
gp_r(gp_row) = %1(gp_row,"%xvalue%");
gp_col("%xvalue%") = no;

$goto write

*       3. X values are in a separate array:

$label xparam
$if  partype %xvalue% $goto gotparam
$abort  Unrecognized symbol specified as xvalue in plot call: %xvalue%
$label gotparam
gp_r(gp_row) = %xvalue%(gp_row);

$label write

put gp_inp;
file.lw = 0;
file.nr = 0;

* Allow user to specify one option before plot is done
$if setglobal gp_opt0  put "%gp_opt0%"/;

put "cd('%gams.wdir%');"/;

$if dimension 2 %1 $goto twodimen
$if not dimension 1 %1 $goto multidim

*       A. Plot a vector.

$if setglobal gp_figure put "gp.figure = figure(%gp_figure%); clf reset;"/;
put "gp.data = load('gnuplot.dat'); gp.data = reshape(gp.data,";
put card(gp_row),",1,2);"/; 
put "gp.plot = plot(gp.data(:,:,1),gp.data(:,:,2));"/;
put "gp.axpos = get(gca,'Position');"/;
put "set(gca,'Position',[gp.axpos(1) gp.axpos(2) %gp_xscale%*gp.axpos(3) %gp_yscale%*gp.axpos(4)]);"/;
put 'legend off'/;

gp_data.nw = 16;
gp_data.nd = 8;
gp_data.nr = 2;

gp_counter = 0;
loop(gp_row,
  if (%1(gp_row) eq gp_na,
    gp_counter = gp_counter + 1;
  else
    put gp_data, gp_r(gp_row)," ",%1(gp_row)/;
  );
  put /;
);
gp_count = 0;
loop(gp_row$(gp_count < gp_counter),
  gp_count = gp_count + 1;
  put gp_data, "NAN NAN"/;
);

$goto putclose

$label twodimen

*       B. Plot a two-dimensional array.

$if setglobal gp_figure put "gp.figure = figure(%gp_figure%); clf reset;"/;
put "gp.data = load('gnuplot.dat'); gp.data = reshape(gp.data,";
put card(gp_row),",",card(gp_col),",2);"/; 
put "gp.plot = plot(gp.data(:,:,1),gp.data(:,:,2));"/;
put "gp.axpos = get(gca,'Position');"/;
put "set(gca,'Position',[gp.axpos(1) gp.axpos(2) %gp_xscale%*gp.axpos(3) %gp_yscale%*gp.axpos(4)]);"/;
put 'legend(gp.plot,'; 
file.nw = 0
loop(gp_col, 
  put  "'",gp_col.te(gp_col),"',";
); 
put '%gp_key%);'/;
file.nw = 6;

gp_data.nw = 16;
gp_data.nd = 8;
gp_data.nr = 2;

loop(gp_col,
  gp_counter = 0;
  loop(gp_row,
      if (%1(gp_row,gp_col) eq gp_na,
        gp_counter = gp_counter + 1;
      else
        put gp_data, gp_r(gp_row)," ",%1(gp_row,gp_col)/;
      );
    );
    gp_count = 0;
    loop(gp_row$(gp_count < gp_counter),
      gp_count = gp_count + 1;
      put gp_data, "NAN NAN"/;
    );
);
$goto putclose


$label multidim

*       C. Plot a multi-dimensional array

$if setglobal gp_figure put "gp.figure = figure(%gp_figure%); clf reset;"/;
put "gp.data = load('gnuplot.dat'); gp.data = reshape(gp.data,";
put card(gp_row),",",card(%series%),",2);"/; 
put "gp.plot = plot(gp.data(:,:,1),gp.data(:,:,2));"/;
put "gp.axpos = get(gca,'Position');"/;
put "set(gca,'Position',[gp.axpos(1) gp.axpos(2) %gp_xscale%*gp.axpos(3) %gp_yscale%*gp.axpos(4)]);"/;
put 'legend(gp.plot,'; 
file.nw = 0
loop(%series%,
  put  "'",%series%.te(%series%),"',";
); 
put '%gp_key%);'/;

gp_data.nw = 16;
gp_data.nd = 8;
gp_data.nr = 2;

loop(%series%,
  gp_counter = 0;
  loop(gp_row,
      if (%1(gp_row,%series%) eq gp_na,
        gp_counter = gp_counter + 1;
      else
        put gp_data, gp_r(gp_row)," ",%1(gp_row,%series%)/;
      );
    );
    gp_count = 0;
    loop(gp_row$(gp_count < gp_counter),
      gp_count = gp_count + 1;
      put gp_data, "NAN NAN"/;
    );
);

$label putclose 
putclose;

* reorder this from GNUPLOT routine so options come after plot

put gp_inp;
file.lw = 0;
file.nr = 0;
file.nd = 0;
file.nw = 6;

put "title('",%1.ts,"');"/;

put "gp.axis = axis; gp.axis(1:2) = [";
file.nd = 0; file.nw = 6;
$if not setglobal xvalue put "1 ",card(gp_row);

file.nd = 3; file.nw = 12;
$if setglobal xvalue put (smin(gp_row,gp_r(gp_row)))," ",(smax(gp_row,gp_r(gp_row)));
put "]; axis(gp.axis);"/;

$if not setglobal labels $goto options

file.nd = 3;
file.nw = 12;
put 'gp.tick = [';
gp_count = 0;
gp_counter = 0;
loop(gp_row,
  gp_count=gp_count+1; 
  if (%labels%(gp_row), 
    gp_counter = gp_counter + 1;
    if (gp_counter gt 1, put ';';);
$if not setglobal xvalue  put gp_count;
$if     setglobal xvalue  put %xvalue%(gp_row);
  );
); 
put '];'/;
put "set(gca,'XTick',gp.tick);"/;
put 'gp.ticklabel = strvcat(';
gp_count = 0;
gp_counter = 0;
loop(gp_row,
  gp_count=gp_count+1; 
  if (%labels%(gp_row), 
    gp_counter = gp_counter + 1;
    if (gp_counter gt 1, put ',';);
    put "'",%labels%.te(gp_row),"'";
  );
); 
put ');'/;
put "set(gca,'XTickLabel',gp.ticklabel);"/;

*       Pass through any user-specified options:

$label options

put "set(gp.plot,'LineStyle','-');"/;

*       Permit user-override of up to 20 PLOT features:

$if setglobal gp_opt1  put "%gp_opt1%"/;
$if setglobal gp_opt2  put "%gp_opt2%"/;
$if setglobal gp_opt3  put "%gp_opt3%"/;
$if setglobal gp_opt4  put "%gp_opt4%"/;
$if setglobal gp_opt5  put "%gp_opt5%"/;
$if setglobal gp_opt6  put "%gp_opt6%"/;
$if setglobal gp_opt7  put "%gp_opt7%"/;
$if setglobal gp_opt8  put "%gp_opt8%"/;
$if setglobal gp_opt9  put "%gp_opt9%"/;


$if setglobal gp_opt10  put "%gp_opt10%"/;
$if setglobal gp_opt11  put "%gp_opt11%"/;
$if setglobal gp_opt12  put "%gp_opt12%"/;
$if setglobal gp_opt13  put "%gp_opt13%"/;
$if setglobal gp_opt14  put "%gp_opt14%"/;
$if setglobal gp_opt15  put "%gp_opt15%"/;
$if setglobal gp_opt16  put "%gp_opt16%"/;
$if setglobal gp_opt17  put "%gp_opt17%"/;
$if setglobal gp_opt18  put "%gp_opt18%"/;
$if setglobal gp_opt19  put "%gp_opt19%"/;

$if not setglobal batch $set batch no
$if %batch%=="no" put "pause,";

putclose /;

*       We are done with the row and column sets.  Clean them
*       out so that they can be used a second time:

gp_row(gp__1) = no;
gp_col(gp__2) = no;

$offuni

$if     %batch%=="no" execute 'matplot %batch%';
$if not %batch%=="no" execute 'matplot';


